#ifndef _LIB_PCX_H_
#define _LIB_PCX_H_

#include "lib_image.h"


#define SIZEOF_PCX_HEADER      128
#define PCX_LAST_VER           5	/* last acceptable version number         */
#define PCX_RLL                1	/* PCX RLL encoding method                */
#define PCX_MAGIC              0x0a	/* first byte of a PCX image              */
#define PCX_256COLORS_MAGIC    0x0c	/* first byte of a PCX 256 color map      */
#define PCX_MAXDEPTH           8	/* PCX supports up to 8 bits per pixel    */
#define PCX_MAXCOLORS          256	/* maximum number of colors in a PCX file */


struct PCX_HEADER
{
   unchar signature;             /* PCX file identifier                 */
   unchar version;               /* version compatibility level         */
   unchar encoding;              /* encoding method                     */
   unchar bits_per_pixel;        /* bits per pixel, or depth            */
   ushort xmin;                  /* X position of left edge             */
   ushort ymin;                  /* Y position of top edge              */
   ushort xmax;                  /* X position of right edge            */
   ushort ymax;                  /* Y position of bottom edge           */
   ushort hres;                  /* X screen resolution of source image */
   ushort vres;                  /* Y screen resolution of source image */
   unchar palette16[16][3];      /* PCX color map                       */
   unchar reserved;              /* should be 0, 1 if std res fax       */
   unchar color_planes;          /* bit planes in image                 */
   ushort bytes_per_line;        /* byte delta between scanlines        */
   ushort palette_type;          /* 0 = undef, 1 = color, 2 = grayscale */
   unchar filler[58];            /* fill to struct size of 128          */
};


#if defined (__STDC__) || defined (__cplusplus) || defined (NeedFunctionPrototypes)
#  define __PARMS(s) s
#else
#  define __PARMS(s) ()
#endif

struct IMAGE_DATA *pcx_image_load     __PARMS((char *filename));
image_t           *pcx_get_image_part __PARMS((struct IMAGE_DATA *image, short x_offset, short y_offset, short width, short height));
short              pcx_image_save     __PARMS((struct IMAGE_DATA *image, char *filename));

#undef __PARMS
#endif /* _LIB_PCX_H_ */
