/***
 * Copyright (C) 1996-98. by Miroslav Zagorac <zaga@tirnanog.cc.fer.hr>
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/param.h>

#include "../data/objects_def.h"
#include "../lib/lib_debug.h"

#include "cave.h"
#include "cave_data.h"
#include "commodore.h"
#include "graphics.h"
#include "help.h"
#include "keyboard.h"
#include "level_editor.h"
#include "objects.h"
#include "main.h"
#include "spectrum.h"


static struct
{
   ushort  key_code;
   unchar  object;
   unchar *image;
} object_table[] =
   {
      { KEY_EDITOR_1,       SPACE, NULL },
      { KEY_EDITOR_2,        DIRT, NULL },
      { KEY_EDITOR_3,        WALL, NULL },
      { KEY_EDITOR_4,     DIAMOND, NULL },
      { KEY_EDITOR_5,   FIREFLY_1, NULL },
      { KEY_EDITOR_6, BUTTERFLY_1, NULL },
      { KEY_EDITOR_7,     BOULDER, NULL },
      { KEY_EDITOR_8,  BRICK_WALL, NULL },
      { KEY_EDITOR_9,  MAGIC_WALL, NULL },
      { KEY_EDITOR_0,      AMOEBA, NULL },
      { KEY_EDITOR_R,    ROCKFORD, NULL },
      { KEY_EDITOR_E,  PRE_OUTBOX, NULL },
      {            0,           0, NULL }
   };
static struct COORDINATE offset, cursor;
static struct CAVE_DATA  edit_cave;
static short             cursor_counter, overwrite_mode, parameter_edit, color_edit, undo_level;


#ifdef ANSI_FUNC

static inline short in_cave (short x, short y)
#else

static inline short in_cave (x, y)
   short x;
   short y;
#endif
{
   return ((x >= 0) && (x < CAVE_WIDTH) && (y >= 0) && (y < CAVE_HEIGHT));
}


#ifdef ANSI_FUNC

static unchar *object_image (unchar object)
#else

static unchar *object_image (object)
   unchar object;
#endif
{
   short i;

   debug (("{%s(%d)", __FUNCTION__, object))

   object = (object == PRE_ROCKFORD_1) ? ROCKFORD : object;

   for (i = 0; object_table[i].key_code; i++)
      if (object_table[i].object == object)
      {
         object = (object == PRE_ROCKFORD_1) ? ROCKFORD : object;

         if (! object_table[i].image)
         {
            struct OBJECT_DATA *object_data = get_object (object);

            if ((object_table[i].image = (unchar *) malloc (16 * 16)))
               graph_scalebox (32, 32, object_data->image[0], 16, 16, object_table[i].image);
         }

         break;
      }

   if (! object_table[i].key_code)
   {
      if ((object_table[i].image = (unchar *) malloc (16 * 16)))
         graph_scalebox (32, 32, unknown, 16, 16, object_table[i].image);
   }

   debug_return ("} == %d", object_table[i].image)
}


#ifdef ANSI_FUNC

static void object_images_free (void)
#else
   
static void object_images_free ()
#endif
{
   short i = 0;

   debug (("{%s()", __FUNCTION__))

   do
   {
      if (object_table[i].image)
         free (object_table[i].image);

      object_table[i].image = NULL;
   }
   while (object_table[i++].key_code);

   debug_endfunc
}


#ifdef ANSI_FUNC

static void level_show (void)
#else

static void level_show ()
#endif
{
   short x, y;

   debug (("{%s()", __FUNCTION__))

   offset.x += (cursor.x < offset.x) ? -(offset.x - cursor.x) : ((cursor.x >= (offset.x + LEVEL_VWIDTH)) ? (cursor.x - offset.x - LEVEL_VWIDTH + 1) : 0);
   offset.y += (cursor.y < offset.y) ? -(offset.y - cursor.y) : ((cursor.y >= (offset.y + LEVEL_VHEIGHT)) ? (cursor.y - offset.y - LEVEL_VHEIGHT + 1) : 0);

   for (x = 0; x < LEVEL_VWIDTH; x++)
      for (y = 0; y < LEVEL_VHEIGHT; y++)
      {
         short position = CAVE_POSITION (offset.x + x, offset.y + y);
         short real_x = x + offset.x, real_y = y + offset.y;

         if (!parameter_edit && !color_edit && ((real_x == cursor.x) && (real_y == cursor.y) && !(cursor_counter++ & 0x08)))
            graph_putbox (x * 16, (y + 2.5) * 16, 16, 16, overwrite_mode ? cursor_overwrite : cursor_normal);
         else
            graph_putbox (x * 16, (y + 2.5) * 16, 16, 16, object_image (edit_cave.data[position]));
      }

   debug_endfunc
}


#ifdef ANSI_FUNC

static void undo (short copy_flag)
#else

static void undo (copy_flag)
   short copy_flag;
#endif
{
   static struct CAVE_DATA  undo_cave[MAX_UNDO_LEVELS];
   static struct COORDINATE undo_offset[MAX_UNDO_LEVELS], undo_cursor[MAX_UNDO_LEVELS];

   debug (("{%s(%d)", __FUNCTION__, copy_flag))

   if (copy_flag & UNDO_PUSH)
   {
      /***
       * Kopiranje u undo buffer, i to samo ako se nesto promijenilo od zadnjeg poziva.
       */
      if (!undo_level || memcmp (&edit_cave, &undo_cave[undo_level - 1], sizeof (struct CAVE_DATA)))
      {
         if (++undo_level > MAX_UNDO_LEVELS)
         {
            short i;

            for (i = 1; i < MAX_UNDO_LEVELS; i++)
            {
               (void) memcpy (&undo_cave[i - 1], &undo_cave[i], sizeof (struct CAVE_DATA));
               (void) memcpy (&undo_offset[i - 1], &undo_offset[i], sizeof (struct COORDINATE));
               (void) memcpy (&undo_cursor[i - 1], &undo_cursor[i], sizeof (struct COORDINATE));
            }

            undo_level--;
         }

         (void) memcpy (&undo_cave[undo_level - 1], &edit_cave, sizeof (struct CAVE_DATA));
         (void) memcpy (&undo_offset[undo_level - 1], &offset, sizeof (struct COORDINATE));
         (void) memcpy (&undo_cursor[undo_level - 1], &cursor, sizeof (struct COORDINATE));
      }
   }
   else if (copy_flag & UNDO_CHECK_LAST)
   {
      /***
       * Brisanje jednog nivoa undo buffera ako se nista nije promijenilo.
       */
      if (undo_level && !memcmp (&edit_cave, &undo_cave[undo_level - 1], sizeof (struct CAVE_DATA)))
         undo_level--;
   }
   else if ((copy_flag & UNDO_POP) && undo_level && memcmp (&edit_cave, &undo_cave[--undo_level], sizeof (struct CAVE_DATA)))
   {
      /***
       * Vracanje sadrzaja iz undo buffera.
       */
      (void) memcpy (&edit_cave, &undo_cave[undo_level], sizeof (struct CAVE_DATA));
      (void) memcpy (&offset, &undo_offset[undo_level], sizeof (struct COORDINATE));
      (void) memcpy (&cursor, &undo_cursor[undo_level], sizeof (struct COORDINATE));
   }

   debug_endfunc
}


#ifdef ANSI_FUNC

char *code_level (short level)
#else
   
char *code_level (level)
   short room;
#endif
{
   static char retval[3] = "A1";

   retval[0] = level / 5 + 'A';
   retval[1] = (level % 5) + '1';

   return (retval);
}


#ifdef ANSI_FUNC

char code_room (short room)
#else
   
char code_room (room)
   short room;
#endif
{
   return ((room < CAVE_MAXROOM) ? (room + 'A') : (room - CAVE_MAXROOM + '1'));
}


#ifdef ANSI_FUNC

char *level_name (short room, short level)
#else

char *level_name (room, level)
   short room;
   short level;
#endif
{
   static char _name[MAXPATHLEN];

   (void) sprintf (_name, "%s/level_%c-%s", gamelibdir, code_room (room), code_level (level));

   return (_name);
}


#ifdef ANSI_FUNC

static short level_load_file (short room, short level, struct CAVE_DATA *cave_data)
#else

static short level_load_file (room, level, cave_data)
   short room;
   short level;
   struct CAVE_DATA *cave_data;
#endif
{
   int level_fd;

   debug (("{%s(%d, %d, 0x%08lx)", __FUNCTION__, room, level, (long) cave_data))

   if ((level_fd = open (level_name (room, level), O_RDONLY)) < 0)
      debug_return ("} == %d", -errno)

   if (read (level_fd, cave_data, sizeof (struct CAVE_DATA)) != sizeof (struct CAVE_DATA))
   {
      (void) close (level_fd);

      debug_return ("} == %d", -errno)
   }

   (void) close (level_fd);

   debug_return ("} == %d", FUNC_RET_OK)
}


#ifdef ANSI_FUNC

short level_load (short room, short level, struct CAVE_DATA *cave_data)
#else

short level_load (room, level, cave_data)
   short room;
   short level;
   struct CAVE_DATA *cave_data;
#endif
{
   short retval;

   debug (("{%s(%d, %d, 0x%08lx)\n", __FUNCTION__, room, level, (long) cave_data))

   if ((retval = level_load_file (room, level, cave_data)) != FUNC_RET_OK)
      debug_return ("} == %d", cave_decode (room, level, cave_data))

   debug_return ("} == %d", retval)
}


#ifdef ANSI_FUNC

short level_save (short room, short level, struct CAVE_DATA *cave_data)
#else

short level_save (room, level, cave_data)
   short room;
   short level;
   struct CAVE_DATA *cave_data;
#endif
{
   int level_fd;

   debug (("{%s(%d, %d, 0x%08lx)\n", __FUNCTION__, room, level, (long) cave_data))

   if ((level_fd = open (level_name (room, level), O_RDWR | O_CREAT, 0644)) < 0)
      debug_return ("} == %d", -errno)

   if (write (level_fd, cave_data, sizeof (struct CAVE_DATA)) != sizeof (struct CAVE_DATA))
   {
      (void) close (level_fd);

      debug_return ("} == %d", -errno)
   }

   (void) close (level_fd);

   debug_return ("} == %d", FUNC_RET_OK)
}


#ifdef ANSI_FUNC

static void level_clear (void)
#else

static void level_clear ()
#endif
{
   short i;

   debug (("{%s()", __FUNCTION__))

   undo (UNDO_PUSH);

   (void) memset (edit_cave.data, DIRT, CAVE_SIZE);
   (void) memset (edit_cave.data, WALL, CAVE_WIDTH);
   (void) memset (edit_cave.data + CAVE_SIZE - CAVE_WIDTH, WALL, CAVE_WIDTH);

   for (i = 1; i < (CAVE_HEIGHT - 1); i++)
      edit_cave.data[CAVE_POSITION (0, i)] = edit_cave.data[CAVE_POSITION (-1, i + 1)] = WALL;

   debug_endfunc
}


#ifdef ANSI_FUNC

static void mark_object (short object)
#else

static void mark_object (object)
   short object;
#endif
{
   short x, y, i;

   debug (("{%s(%d)", __FUNCTION__, object))

   for (i = 0; object_table[i].object != object; i++);

   x = 35 * CHAR_16x16 + 7;
   y = (2 + i) * (CHAR_16x16 + 7);

   graph_fillbox (x, y, 22, 18, WHITE);
   graph_putbox (x + 5, y + 1, CHAR_16x16, CHAR_16x16, object_image (object_table[i].object));

   debug_endfunc
}


#ifdef ANSI_FUNC

static void colors_show (void)
#else
   
static void colors_show ()
#endif
{
   static short counter = 0;
   short        i, x = 35 * CHAR_16x16 + 10, y = 26.5 * CHAR_16x16;

   for (i = 0; i < 3; i++)
   {
      if ((i + 1) == color_edit)
         graph_rectangle (x + i * (CHAR_16x16 + 6) - 2, y - 2, 20, 20, (++counter & 0x08) ? GREEN : DARKGRAY);

      graph_fillbox (x + i * (CHAR_16x16 + 6), y, 16, 16, edit_cave.color[i] + PALETTE_COMMODORE_OFFSET);
   }
}


#ifdef ANSI_FUNC

static short count_objects (short object)
#else

static short count_objects (object)
   short object;
#endif
{
   short i, retval = 0;

   debug (("{%s(%d)", __FUNCTION__, object))

   for (i = 0; i < CAVE_SIZE; i++)
      if (edit_cave.data[i] == object)
         retval++;

   debug_return ("} == %d", retval)
}


#ifdef ANSI_FUNC

static void top_print (char room, short level)
#else

static void top_print (room, level)
   char room;
   short level;
#endif
{
   static short counter = 0;

   graph_printf (13 * CHAR_16x16, 0.5 * CHAR_16x16, LIGHTGRAY, bd_font_16x16, "-         ROOM  /");
   graph_printf (10 * CHAR_16x16, 0.5 * CHAR_16x16, WHITE, bd_font_16x16, "\023");
   graph_printf (7 * CHAR_16x16, 0.5 * CHAR_16x16, (parameter_edit == 1) ? ((++counter & 0x08) ? RED : DARKGRAY) : RED, bd_font_16x16, "%03d", edit_cave.diamonds);
   graph_printf (11 * CHAR_16x16, 0.5 * CHAR_16x16, (parameter_edit == 2) ? ((++counter & 0x08) ? LIGHTGRAY : DARKGRAY) : LIGHTGRAY, bd_font_16x16, "%02d", edit_cave.dvalue_init);
   graph_printf (14 * CHAR_16x16, 0.5 * CHAR_16x16, (parameter_edit == 3) ? ((++counter & 0x08) ? LIGHTGRAY : DARKGRAY) : LIGHTGRAY, bd_font_16x16, "%02d", edit_cave.dvalue_extra);
   graph_printf (18 * CHAR_16x16, 0.5 * CHAR_16x16, (parameter_edit == 4) ? ((++counter & 0x08) ? LIGHTGRAY : DARKGRAY) : LIGHTGRAY, bd_font_16x16, "%03ld", edit_cave.time / 1000000L);
   graph_printf (28 * CHAR_16x16, 0.5 * CHAR_16x16, (parameter_edit == 5) ? ((++counter & 0x08) ? LIGHTGRAY : DARKGRAY) : LIGHTGRAY, bd_font_16x16, "%c", code_room (room));
   graph_printf (30 * CHAR_16x16, 0.5 * CHAR_16x16, (parameter_edit == 6) ? ((++counter & 0x08) ? LIGHTGRAY : DARKGRAY) : LIGHTGRAY, bd_font_16x16, "%s", code_level (level));
}


/***
 * SHIFT + C             - Clear level.
 * SHIFT + L             - Load level.
 * SHIFT + S             - Save level.
 * SHIFT + Left/Right    - Up/down room letter.
 * SHIFT + Up/Down       - Up/down room level.
 * D     + Up/Down       - Set number of diamonds.
 * S     + Up/Down       - Set initial value for each diamond.
 * T     + Up/Down       - Set game time.
 * Left, Right, Up, Down - Move cursor.
 */
#ifdef ANSI_FUNC

void level_editor (void)
#else

void level_editor ()
#endif
{
   short selected_object, repeat_counter = 0, init_flag = 0, i;

   debug (("{%s()", __FUNCTION__))

   cursor_counter = 0;
   overwrite_mode = 0;
   parameter_edit = 0;
   color_edit     = 0;
   undo_level     = 0;

   offset.x = offset.y = 0;
   cursor.x = cursor.y = 0;

   graph_palette_fade (-4);
   (void) key_pressed_code (0, 0);

   if (level_load (game_data.start_room, game_data.start_level, &edit_cave) != FUNC_RET_OK)
      level_clear ();

   /***
    * To je objekt koji se nalazi ispod kursora.
    */
   selected_object = edit_cave.data[CAVE_POSITION (cursor.x, cursor.y)];

   show_refresh_rate (1);
   game_timer (1);

   while (1)
   {
      short x = 0, y = 0;

      if (! parameter_edit)
         undo (UNDO_CHECK_LAST);

      game_timer (0);
      (void) key_update ();

      graph_clearscreen (BLACK);
      top_print (edit_cave.room, edit_cave.level);
      graph_putbox (35 * CHAR_16x16 + 1, 2.5 * CHAR_16x16, 80, 440, panel);

      for (i = 0; object_table[i].key_code; i++)
      {
         graph_putbox (35 * CHAR_16x16 + 12, (2 + i) * (CHAR_16x16 + 7) + 1, CHAR_16x16, CHAR_16x16, object_image (object_table[i].object));
         graph_printf (37.25 * CHAR_16x16, (2 + i) * (CHAR_16x16 + 7) + 4, GREEN, bd_font_8x8, "%04d", count_objects (object_table[i].object));
      }

      graph_printf (70.75 * CHAR_8x8, 56.5 * CHAR_8x8, RED, bd_font_8x8, "Undo:%d", undo_level);
      graph_printf (70.75 * CHAR_8x8, 57.75 * CHAR_8x8, RED, bd_font_8x8, "XY:%02d,%02d", cursor.x + 1, cursor.y + 1);

      mark_object (selected_object);
      colors_show ();
      level_show ();

      if (key_pressed_code (KEY_HELP, 1))
         show_editor_help (1);
      else
         show_editor_help (-1);

      show_refresh_rate (0);
      graph_copyscreen ();

      if (! init_flag)
         graph_palette_fade (init_flag = 4);

      if (parameter_edit)
      {
         /***
          * Mijenjanje vrijednosti parametara nivoa (gornji red na ekranu).
          */
         if (key_pressed_code (KEY_EDITOR_RIGHT, 0))
            parameter_edit += (parameter_edit < 6) ? 1 : (-parameter_edit + 1);
         else if (key_pressed_code (KEY_EDITOR_LEFT, 0))
            parameter_edit -= (parameter_edit > 1) ? 1 : -5;
         else if (key_pressed_code (KEY_ENTER, 0))
            parameter_edit = 0;
         else if (game_data.game_status & GAME_OBJECT_ANIMATE)
         {
            static short cave_time;
            static struct
            {
               short  min_value;
               short  max_value;
               short *addr;
            } values[] =
               {
                  { 1,                 255, &edit_cave.diamonds     },
                  { 1,                  99, &edit_cave.dvalue_init  },
                  { 0,                  99, &edit_cave.dvalue_extra },
                  { 1,                 255, &cave_time              },
                  { 0, CAVE_MAXROOMEXT - 1, &edit_cave.room         },
                  { 0,   CAVE_MAXLEVEL - 1, &edit_cave.level        }
               };

            cave_time = edit_cave.time / 1000000L;

            if (key_pressed_code (KEY_EDITOR_UP, 1))
            {
               if (!repeat_counter++ || (repeat_counter > 4))
                  *values[parameter_edit - 1].addr += (*values[parameter_edit - 1].addr < values[parameter_edit - 1].max_value) ? 1 : 0;
            }
            else if (key_pressed_code (KEY_EDITOR_DOWN, 1))
            {
               if (!repeat_counter++ || (repeat_counter > 4))
                  *values[parameter_edit - 1].addr -= (*values[parameter_edit - 1].addr > values[parameter_edit - 1].min_value) ? 1 : 0;
            }
            else
               repeat_counter = 0;

            edit_cave.time         = cave_time * 1000000L;
            game_data.game_status &= ~GAME_OBJECT_ANIMATE;
         }
      }
      else if (color_edit)
      {
         /***
          * Editiranje boja objekata.
          */
         if (key_pressed_code (KEY_EDITOR_RIGHT, 0))
            color_edit += (color_edit < 3) ? 1 : (-color_edit + 1);
         else if (key_pressed_code (KEY_EDITOR_LEFT, 0))
            color_edit -= (color_edit > 1) ? 1 : -2;
         else if (key_pressed_code (KEY_ENTER, 0))
            color_edit = 0;
         else if (game_data.game_status & GAME_OBJECT_ANIMATE)
         {
            struct CAVE_RAW_DATA cave_raw;

            if (key_pressed_code (KEY_EDITOR_UP, 1))
            {
               if (!repeat_counter++ || (repeat_counter > 4))
                  edit_cave.color[color_edit - 1] = (edit_cave.color[color_edit - 1] + 1) & 0x0f;
            }
            else if (key_pressed_code (KEY_EDITOR_DOWN, 1))
            {
               if (!repeat_counter++ || (repeat_counter > 4))
                  edit_cave.color[color_edit - 1] = (edit_cave.color[color_edit - 1] - 1) & 0x0f;
            }
            else
               repeat_counter = 0;

            object_images_free ();

            if (TEST_OPTION (OPT_COMMODORE))
            {
               cave_raw.bg1_color = edit_cave.color[0];
               cave_raw.bg2_color = edit_cave.color[1];
               cave_raw.fg_color  = edit_cave.color[2];

               commodore_change_colors (&cave_raw);
            }
            else if (TEST_OPTION (OPT_SPECTRUM))
            {
               cave_raw.bg1_color = edit_cave.color[0];
               cave_raw.bg2_color = edit_cave.color[1];
               cave_raw.fg_color  = edit_cave.color[2];

               spectrum_change_colors (&cave_raw);
            }

            game_data.game_status &= ~GAME_OBJECT_ANIMATE;
         }
      }
      else
      {
         if (key_pressed_code (KEY_ENTER, 0))
         {
            undo (UNDO_PUSH);

            parameter_edit = 1;
         }
         else if (key_pressed_code (KEY_EDITOR_COLORS, 0))
         {
            undo (UNDO_PUSH);

            color_edit = 1;
         }
         else if (key_pressed_code (KEY_EDITOR_CLEAR, 0))
            level_clear ();
         else if (key_pressed_code (KEY_EDITOR_UNDO, 0))
         {
            undo (UNDO_POP);

            if (overwrite_mode)
               undo (UNDO_PUSH);
         }
         else if (key_pressed_code (KEY_EDITOR_LOAD, 0))
         {
            object_images_free ();
            undo (UNDO_PUSH);

            if (level_load (edit_cave.room, edit_cave.level, &edit_cave) != FUNC_RET_OK)
               level_clear ();
         }
         else if (key_pressed_code (KEY_EDITOR_SAVE, 0))
         {
            (void) level_save (edit_cave.room, edit_cave.level, &edit_cave);
         }
         else if (key_pressed_code (KEY_EDITOR_QUIT, 0))
         {
            graph_palette_fade (-4);
            object_images_free ();

            break;
         }

         if (game_data.game_status & GAME_OBJECT_ANIMATE)
         {
            if (key_pressed_code (KEY_EDITOR_INSERT, 0))
            {
               if ((overwrite_mode ^= 1))
                  undo (UNDO_PUSH);
               else
                  undo (UNDO_CHECK_LAST);
            }
            else if (key_pressed_code (KEY_EDITOR_HOME, 0))
               x -= cursor.x;
            else if (key_pressed_code (KEY_EDITOR_END, 0))
               x += CAVE_WIDTH - cursor.x - 1;
            else if (key_pressed_code (KEY_EDITOR_TOP, 0))
               y -= cursor.y;
            else if (key_pressed_code (KEY_EDITOR_BOTTOM, 0))
               y += CAVE_HEIGHT - cursor.y - 1;
#ifdef DEBUG
            else if (key_pressed_code (KEY_SNAPSHOT, 0))
               (void) graph_save_snapshot ();
#endif
            else
            {
               /***
                * Pomicanje kursora po jedno polje.
                */
               if (key_pressed_code (KEY_EDITOR_LEFT, 1))
                  x = -1;
               else if (key_pressed_code (KEY_EDITOR_RIGHT, 1))
                  x = 1;

               if (key_pressed_code (KEY_EDITOR_UP, 1))
                  y = -1;
               else if (key_pressed_code (KEY_EDITOR_DOWN, 1))
                  y = 1;
            }

            /***
             * Ispitivanje za promjenu objekta.
             */
            for (i = 0; object_table[i].key_code; i++)
               if (key_pressed_code (object_table[i].key_code, 0))
               {
                  if (selected_object != object_table[i].object)
                  {
                     undo (UNDO_PUSH);

                     edit_cave.data[CAVE_POSITION (cursor.x, cursor.y)] = selected_object = object_table[i].object;
                  }

                  break;
               }

            if (x || y)
            {
               if ((!repeat_counter++ || (repeat_counter > 4)) && in_cave (cursor.x + x, cursor.y + y))
               {
                  cursor.x += x;
                  cursor.y += y;

                  if (overwrite_mode)
                     edit_cave.data[CAVE_POSITION (cursor.x, cursor.y)] = (unchar) selected_object;
                  else
                     selected_object = edit_cave.data[CAVE_POSITION (cursor.x, cursor.y)];

                  cursor_counter = 0;
               }
            }
            else
               repeat_counter = 0;

            game_data.game_status &= ~GAME_OBJECT_ANIMATE;
         }
      }
   }

   debug_endfunc
}
